@@ -2,6 +2,8 @@ import json
 import pathlib
 import random
 import shutil
+import uuid
+import warnings
 from io import BytesIO, StringIO
 from itertools import groupby
 from typing import Optional, Dict
@@ -160,7 +162,10 @@ class COCOCreator:
                 if ann.class_name in map_dict:
                     ann.class_name = map_dict[ann.class_name]
         for old, new in map_dict.items():
-            self.classes.remove(old)
+            if old in self.classes:
+                self.classes.remove(old)
+            else:
+                warnings.warn(f"Class '{old}' does not exist. Adding new class '{new}' anyway.")
             self.classes.add(new)
 
     def remove_classes(self, classes):
@@ -187,6 +192,13 @@ class COCOCreator:
         self.__image_path_ann_lut[image_path].append(annotation)
         self.classes.add(class_name)
 
+    def add_detection_image(self, image_path):
+        if self.__mode != CocoModes.DETECTION:
+            raise ValueError(f"Cannot call add_detection_annotation when creating a {self.__mode} coco file")
+        image_path = str(image_path)  # Ensure it's a string not Path object
+        if image_path not in self.__image_path_ann_lut:
+            self.__image_path_ann_lut[image_path] = []
+
     def add_images_with_masks(self, image_list, mask_list, classes_list):
         """Adds detections from an image and corresponding binary mask"""
         for image_path, mask_path, mask_class in zip(image_list, mask_list, classes_list):
@@ -233,6 +245,35 @@ class COCOCreator:
             file_name = image_root / image_id_to_filename[annotation["image_id"]]
             self.add_detection_annotation(file_name, class_name, bbox, segm)
 
+    def add_darwin_v7(self, json_path):
+        """Adds detections from a V7 darwin json file"""
+        if isinstance(json_path, str):
+            json_path = pathlib.Path(json_path)
+        with json_path.open("rb") as fh:
+            darwin = json.load(fh)
+
+        file_id_to_name = {str(idx): d for idx, d in enumerate(darwin["image"]["frame_urls"])}
+        for path in file_id_to_name.values():  # add all images in-case of no annotations for image
+            self.add_detection_image(path)
+
+        annotations = {}  # frame_id: [annotations]
+        for ann in darwin["annotations"]:
+            class_name = ann["name"]
+            interpolated = ann["interpolated"]
+            # validate all instance_ids are consistent
+            ids = set([ii["instance_id"]["value"] for ii in ann["frames"].values() if "instance_id" in ii])
+            if not ids:
+                continue
+            if len(ids) > 1:
+                raise ValueError("Instance IDS not unique")
+            instance_id = ids.pop()
+            for frame_id, frame_annotation in ann["frames"].items():
+                bbox = [frame_annotation["bounding_box"].get(a) for a in ["x", "y", "w", "h"]]
+                if frame_id not in annotations:
+                    annotations[frame_id] = []
+                self.add_detection_annotation(image_path=file_id_to_name[frame_id], bbox=bbox, class_name=class_name,
+                                              kwargs=dict(instance_id=instance_id, interpolated=interpolated,))
+
     def add_label_box_export_async(self, json_path, num_workers=64):
         """Converts a LabelBox JSON (https://app.labelbox.com/) instance segmentation file to COCO"""
         async def get_label_box_async(json_file_path):
@@ -340,12 +381,12 @@ class COCOCreator:
         for field_name, field_info in kwargs.items():
             self.__extra_fields_lut[image_path][field_name] = field_info
 
-    def export(self, dataset_root: pathlib.Path):
+    def export(self, dataset_root: pathlib.Path, *args, **kwargs):
         if self.__mode == CocoModes.DETECTION:
-            return self.export_detection(dataset_root)
+            return self.export_detection(dataset_root, *args, **kwargs)
         raise NotImplementedError(f"Mode  '{self.__mode}' is not currently supported.")
 
-    def export_detection(self, output_dir: pathlib.Path):
+    def export_detection(self, output_dir: pathlib.Path, symlink_to_image: bool = False, exists_ok: bool = False):
         if isinstance(output_dir, str):
             output_dir = pathlib.Path(output_dir)
         output_dir.mkdir(exist_ok=True, parents=True)
@@ -355,15 +396,19 @@ class COCOCreator:
 
         image_path_lut = self.__image_path_ann_lut
         packaged_image_path_lut = {}
-        progress_bar_post_fix = "Copying image data '{}'"
+        progress_bar_post_fix = ("Creating symlink to" if symlink_to_image else "Copying") + " image data '{}'"
         progress_bar = tqdm(total=len(image_path_lut), postfix=progress_bar_post_fix.format(""))
 
+        images_not_local = False
+        tmp_dir = f"/tmp/coco_creator/{uuid.uuid4()}"
+
         # Write image data to new packaged dataset
         for image_file, image_annotation in image_path_lut.items():
             original_file = image_file
             # Download the file if it doesn't exist locally
-            if "http://" in image_file:
-                tmp_file = pathlib.Path(f"/tmp/coco_creator/{pathlib.Path(image_file).name}")
+            if "http://" in image_file or "https://" in image_file:
+                images_not_local = True
+                tmp_file = pathlib.Path(f"{tmp_dir}/{pathlib.Path(image_file).name}")
                 tmp_file.parent.mkdir(exist_ok=True)
                 with tmp_file.open("wb") as handle:
                     response = requests.get(image_file, stream=True)
@@ -387,7 +432,17 @@ class COCOCreator:
             new_image_file = f"data/{image_id}_{pathlib.Path(image_file).name}"
             new_image_file_location = output_dir / new_image_file
             new_image_file_location.parent.mkdir(exist_ok=True, parents=True)
-            shutil.copy(str(image_file), str(new_image_file_location))
+            if symlink_to_image:  # Symlink if parameter is set else copy the file
+                if images_not_local:
+                    raise IOError("Cannot symlink to images if they're not stored locally")
+                try:
+                    pathlib.Path(new_image_file_location).symlink_to(pathlib.Path(image_file),
+                                                                     target_is_directory=False)
+                except FileExistsError:
+                    if not exists_ok:  # sometimes if combining datasets the file will exist in multiple coco files
+                        raise
+            else:
+                shutil.copy(str(image_file), str(new_image_file_location))
 
             packaged_image_path_lut[new_image_file] = (CocoImage(
                 image_id=image_id,
@@ -414,7 +469,12 @@ class COCOCreator:
         file_path_keys = list(packaged_image_path_lut.keys())
         if self.random_splits:
             random.shuffle(file_path_keys)
-        split_keys = dict(all=file_path_keys)
+        # Always export an _combined.json file unless this is what the user is exporting/only one file exported
+        all_data_splits_file = '_'.join(self.data_splits.keys()) + '_combined'
+        if len(self.data_splits.keys()) > 1 and all_data_splits_file not in self.data_splits:
+            split_keys = {all_data_splits_file: file_path_keys}
+        else:
+            split_keys = dict()
         split_keys.update({data_split_keys[i]: file_path_keys[idx_splits[i]:idx_splits[i + 1]]
                            for i in range(0, len(idx_splits) - 1)})
 
